import { Component, OnInit, ElementRef, ViewChild } from '@angular/core';
import { AllSendTransactionMisService } from '../services/all-send-transaction-mis.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-all-sent-transaction-mis',
  templateUrl: './all-sent-transaction-mis.component.html',
  styleUrls: ['./all-sent-transaction-mis.component.css'],
})
export class AllSentTransactionMisComponent implements OnInit {
  @ViewChild('transactionTable') transactionTable: ElementRef;

  allSentTransaction: any[] = [];
  totalNoOfTransactions: number = 0;
  selectedCountryList: any;
  selectedAgentList: number = 0;
  // fromDate: string = '';
  // toDate: string = '';
  fromDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  selectedReportType: string = 'Detail';
  countryList: any[] = [];
  agentList: any[] = [];
  selectedFileFormat: string = '1';
  showTableAndExport: boolean = false;
  isLoading: boolean = false;

  constructor(private transactionService: AllSendTransactionMisService) {}

  ngOnInit(): void {
    this.getCountryList();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getCountryList() {
    this.transactionService.getRecipientCountryList().subscribe({
      next: (res: any[]) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.Countryid;
        this.onCountryChange(this.selectedCountryList);
      },
      error: (error: any) => {
        console.error('Error fetching country list:', error);
      },
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.transactionService.getAgentListByCountry(countryId).subscribe({
        next: (res: any[]) => {
          this.agentList = res;
          this.selectedAgentList = res?.[0]?.Agent_id;
        },
        error: () => {
          this.agentList = [];
        },
      });
    }
  }

  onAgentChange(agentId: any) {}

  displayByDate() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }
    if (!this.fromDate) {
      alert('Please Select From Date!');
      return;
    }
    if (!this.toDate) {
      alert('Please Select To Date!');
      return;
    }
    this.isLoading = true;

    if (
      this.fromDate &&
      this.toDate &&
      this.selectedCountryList &&
      this.selectedAgentList
    ) {
      this.transactionService
        .getTransactionDetailGrid(
          this.fromDate,
          this.toDate,
          this.selectedAgentList,
          this.selectedCountryList
        )
        .subscribe((res: any[]) => {
          if (res?.length == 0) {
            this.isLoading=false
            alert('No data found.');
            return;
          }
          this.allSentTransaction = res.map((item) => item.Table1[0]);
          this.totalNoOfTransactions = this.allSentTransaction.length;
          this.showTableAndExport = true;

          this.isLoading = false;
        });
    } else {
    }
  }

  exportToExcel() {
    if (!this.transactionTable || !this.transactionTable.nativeElement) {
      console.error('Table element not found');
      return;
    }

    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Agentwise_Transaction_Sent_${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }
}
