import { Component } from '@angular/core';
import { BeneficiaryRegisterDetailsService } from '../services/beneficiary-register-details.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-beneficiary-register-details',
  templateUrl: './beneficiary-register-details.component.html',
  styleUrls: ['./beneficiary-register-details.component.css']
})
export class BeneficiaryRegisterDetailsComponent {
  beneficiaryDetails: any[] = [];
  showTable: boolean = false;
  isLoading: boolean = false;
  selectedFileFormat: string = '1';

  // startDate: string = '';
  // endDate: string = '';
  startDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  endDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  
  beneficiaryType: string = '1';

  constructor(private beneficiaryRegisterService: BeneficiaryRegisterDetailsService) {}

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  fetchData(): void {
    if (this.startDate && this.endDate && this.beneficiaryType) {
      this.isLoading = true;
      this.beneficiaryRegisterService.getBeneficiaryRegistrationData(this.beneficiaryType, this.startDate, this.endDate)
        .subscribe(
          (data: any[]) => {
            this.beneficiaryDetails = data;
            this.showTable = true;
            this.isLoading = false;
          },
          (error: any) => {
            console.error('Error fetching beneficiary registration details:', error);
            this.isLoading = false;
          }
        );
    } else {
      console.error('Please fill in all fields');
    }
  }

  onSubmit(): void {
    this.fetchData();
  }

  exportToExcel(): void {
    if (this.beneficiaryDetails && this.beneficiaryDetails.length > 0) {
      const ws: XLSX.WorkSheet = XLSX.utils.json_to_sheet(this.beneficiaryDetails);
      const wb: XLSX.WorkBook = XLSX.utils.book_new();
      XLSX.utils.book_append_sheet(wb, ws, 'BeneficiaryDetails');
      const fileName = `Beneficiary_Details_${new Date().getTime()}.xlsx`;
      XLSX.writeFile(wb, fileName);
    } else {
      console.error('No data available to export');
    }
  }
}
