import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { DailyRatesService } from '../services/daily-rates.service';
import { getCacheData } from '../utils/helper';
import { CifWiseMarginsService } from '../services/cif-wise-margins.service';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-cif-wise-margins',
  templateUrl: './cif-wise-margins.component.html',
  styleUrls: ['./cif-wise-margins.component.css'],
})
export class CifWiseMarginsComponent {
  @ViewChild('manageCIFWise') manageCIFWiseModal;
  bsModalRef: BsModalRef;

  type: any = 'add';
  isLoading: boolean = false;

  sendingCountries: any = [];
  selectedSendCountry: any = 'All';

  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';

  sendingCurrencies: any = [];
  selectedSendingAgent: any = 'All';

  receivingCurrencies: any = [];
  selectedReceivingAgent: any = 'All';

  cifNumbers: any = [];
  selectedCifNumber: any = 'All';

  modeofPayments: any = [];

  cifWiseMargins: any = [];

  manageCIFWiseForm = new FormGroup({
    MarginPercent: new FormControl('', Validators.required),
    ExchangeSetName: new FormControl('', Validators.required),
    CIFNumber: new FormControl('None', Validators.required),
    ModeOfPayemntId: new FormControl('None', Validators.required),
    PayOutCurrencyId: new FormControl('None', Validators.required),
    PayInCurrencyId: new FormControl('None', Validators.required),
    PayOutCountryId: new FormControl('None', Validators.required),
    PayInCountryId: new FormControl('None', Validators.required),
    ID: new FormControl(),
    Active: new FormControl(),
  });

  constructor(
    private dailyRateService: DailyRatesService,
    private cifWiseService: CifWiseMarginsService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService

  ) {}

  ngOnInit(): void {
    this.getSendingCountry();
    this.getAllCIFWiseMargins();
  }

  getSendingCountry() {
    this.isLoading = true;

    this.cifWiseService
      .getSendingCountry(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.sendingCountries = res;
          this.getSendCurrency(0);
          this.getReceivingCountry(0);
        },
        error: () => {
          this.sendingCountries = [];
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }

  getReceivingCountry(countryId: any = 0) {
    this.cifWiseService
      .getReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
          this.getReceiveCurrency(0);
          this.getModeOfPayment(res?.[0]?.PayOutCountryId);
          this.getCIFNumbers();
        },
        error: () => {
          this.receivingCountries = [];
        },
        complete: () => {},
      });
  }

  getAllCIFWiseMargins(payInCountry: any = 0, payOutCountry: any = 0) {
    this.cifWiseService
      .getAllCIFWiseMargins(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountry,
        payOutCountry
      )
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.cifWiseMargins = res.map((originalObject) => {
            const modifiedObject = {};
            for (const key in originalObject) {
              if (Object.prototype.hasOwnProperty.call(originalObject, key)) {
                const modifiedKey = key.replace(/ /g, '_');
                modifiedObject[modifiedKey] = originalObject[key];
              }
            }

            return modifiedObject;
          });
        },
        error: () => {
          this.cifWiseMargins = [];
        },
        complete: () => {},
      });
  }

  getSendCurrency(countryId: any = 0) {
    this.cifWiseService
      .getSendCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCurrencies = res;
        },
        error: () => {
          this.sendingCurrencies = [];
        },
        complete: () => {},
      });
  }

  getReceiveCurrency(countryId: any = 0) {
    this.cifWiseService
      .getReceiveCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCurrencies = res;
        },
        error: () => {
          this.receivingCurrencies = [];
        },
        complete: () => {},
      });
  }

  getModeOfPayment(payOutCountryId: any = 0) {
    this.cifWiseService
      .getModeOfPayment(
        getCacheData('admin_token'),
        getCacheData('username'),
        payOutCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPayments = res;
        },
        error: () => {
          this.modeofPayments = [];
        },
        complete: () => {},
      });
  }

  getCIFNumbers() {
    this.cifWiseService
      .getCIFNumbers(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.cifNumbers = res;
        },
        error: () => {
          this.cifNumbers = [];
        },
        complete: () => {},
      });
  }

  get f() {
    return this.manageCIFWiseForm.value;
  }

  onSendCountryChange() {
    if (this.selectedSendCountry != 'All') {
      this.getReceivingCountry(this.selectedSendCountry);
    }
  }

  onSendingCountryChange(countryId: any) {
    if (countryId != 'None') {
      this.getSendCurrency(countryId);
      this.getReceivingCountry(countryId);
    }
  }

  onReceiveCountryChange(countryId: any) {
    if (countryId != 'None') {
      this.getReceiveCurrency(countryId);
      this.getModeOfPayment(countryId);
      this.getCIFNumbers();
    }
  }

  onReceiveCurrencyChange(countryId) {
    if (countryId !== 'None') {
      // Ensure data arrays are initialized before accessing them
      if (
        this.sendingCountries &&
        this.sendingCurrencies &&
        this.receivingCountries &&
        this.receivingCurrencies
      ) {
        const sendCountry = this.sendingCountries.find(
          (s) =>
            s.PayInCountryId ==
            this.manageCIFWiseForm.get('PayInCountryId').value
        ).CountryName;

        const sendCurrency = this.sendingCurrencies.find(
          (s) =>
            s.Currency_Id == this.manageCIFWiseForm.get('PayInCurrencyId').value
        ).CurrencyShortName;

        const receiveCountry = this.receivingCountries.find(
          (s) =>
            s.PayOutCountryId ==
            this.manageCIFWiseForm.get('PayOutCountryId').value
        ).CountryName;

        const receiveCurrency = this.receivingCurrencies.find(
          (s) =>
            s.Currency_Id ==
            this.manageCIFWiseForm.get('PayOutCurrencyId').value
        ).CurrencyShortName;

        this.manageCIFWiseForm.patchValue({
          ExchangeSetName: `${sendCountry} (${sendCurrency}) - ${receiveCountry} (${receiveCurrency})`,
        });
      }
    }
  }

  searchCifWiseMargin() {
    let payInCountry: any = 0;
    let payOutCountry: any = 0;

    if (this.selectedSendCountry !== 'All') {
      payInCountry = this.selectedSendCountry;
    }

    if (this.selectedReceivingCountry !== 'All') {
      payOutCountry = this.selectedReceivingCountry;
    }

    this.getAllCIFWiseMargins(payInCountry, payOutCountry);
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openCIFWiseModal(type: any) {
    this.type = type;

    if (type === 'add') {
      this.manageCIFWiseForm.reset({
        PayInCountryId: 'None',
        PayOutCountryId: 'None',
        PayInCurrencyId: 'None',
        PayOutCurrencyId: 'None',
        ModeOfPayemntId: 'None',
        MarginPercent: '',
        CIFNumber: 'None',
        ExchangeSetName: '',
      });
    }

    this.openModal(this.manageCIFWiseModal);
  }

  addCIFWiseMargin() {
    if (!this.manageCIFWiseForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.PayInCountryId == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    if (this.f.PayInCurrencyId == 'None') {
      alert('Please Select Send Currency!');
      return;
    }

    if (this.f.PayOutCountryId == 'None') {
      alert('Please Select Receive Country!');
      return;
    }

    if (this.f.PayOutCurrencyId == 'None') {
      alert('Please Select Receive Currency!');
      return;
    }

    if (this.f.ModeOfPayemntId == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    if (this.f.CIFNumber == 'None') {
      alert('Please Select CIF Number!');
      return;
    }

    if (
      Number(this.f.MarginPercent) <= 0.1 ||
      Number(this.f.MarginPercent) >= 1.5
    ) {
      alert('Margin Percent should not be greater than 100%!');
      return;
    }

    let payload;
    payload = {
      PayInCountryId: this.f.PayInCountryId,
      PayInCurrencyId: this.f.PayInCurrencyId,
      PayOutCountryId: this.f.PayOutCountryId,
      PayOutCurrencyId: this.f.PayOutCurrencyId,
      ModeOfPayemntId: this.f.ModeOfPayemntId,
      CIFNumber: this.f.CIFNumber,
      ExchangeSetName: this.f.ExchangeSetName,
      EnteredBy: getCacheData('G_UserID'),
      MarginPercent: this.f.MarginPercent,
      Active: this.f.Active,
    };

    if (this.type === 'edit') {
      payload = [
        {
          ID: this.f.ID,
          PayInCountryId: this.f.PayInCountryId,
          PayInCurrencyId: this.f.PayInCurrencyId,
          PayOutCountryId: this.f.PayOutCountryId,
          PayOutCurrencyId: this.f.PayOutCurrencyId,
          ModeOfPayemntId: this.f.ModeOfPayemntId,
          CIFNumber: this.f.CIFNumber,
          ExchangeSetName: this.f.ExchangeSetName,
          EnteredBy: getCacheData('G_UserID'),
          MarginPercent: this.f.MarginPercent,
          Active: this.f.Active,
        },
      ];
    }

    const serviceMethod =
      this.type === 'add' ? 'addCIFWiseMargin' : 'updateCIFWiseMargin';

    this.cifWiseService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getSendingCountry();
        this.getAllCIFWiseMargins();
        const message = this.type === 'edit' ? 'updated' : 'added';
        alert(res);
      },
      error: (err) => {
        this.modalService.hide();
        // alert(err?.error?.message || 'Something went wrong');
        this.toastrSvc.error(err?.error?.message)

      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editCIFWiseModal(cifWise: any) {
    this.manageCIFWiseForm.patchValue({
      ID: cifWise?.ID,
      PayInCountryId: cifWise?.SendCountryID,
      PayInCurrencyId: cifWise?.SendCurrencyID,
      PayOutCountryId: cifWise?.ReceCountryID,
      PayOutCurrencyId: cifWise?.ReceCurrencyID,
      ModeOfPayemntId: cifWise?.ModeofPayment_id,
      CIFNumber: cifWise?.CIFNumber,
      ExchangeSetName: cifWise?.Exchange_Set_Name,
      MarginPercent: cifWise?.Margin_Percent,
       Active:  cifWise?.Active,
    });
    this.openCIFWiseModal('edit');
  }

  updateCIFWiseMargin(): void {
    const updatedCIFMargins = this.cifWiseMargins
      .filter((con) => con.isChecked)
      .map((con) => {
        return {
          ID: con?.ID,
          PayInCountryId: con?.SendCountryID,
          PayInCurrencyId: con?.SendCurrencyID,
          PayOutCountryId: con?.ReceCountryID,
          PayOutCurrencyId: con?.ReceCurrencyID,
          ModeOfPayemntId: con?.ModeofPayment_id,
          CIFNumber: con?.CIFNumber,
          ExchangeSetName: con?.Exchange_Set_Name,
          MarginPercent: con?.Margin_Percent,
          EnteredBy: getCacheData('G_UserID'),
          Active: con.Active,
        };
      });
    if (updatedCIFMargins?.length) {
      this.cifWiseService
        .updateCIFWiseMargin(
          getCacheData('admin_token'),
          getCacheData('username'),
          updatedCIFMargins
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
            this.getSendingCountry();
            this.getAllCIFWiseMargins();
          },
          error: (err) => {
            this.modalService.hide();
            // alert(err?.error?.message || 'Something went wrong');
            this.toastrSvc.error(err?.error?.message)

          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
