import { Component, TemplateRef, ViewChild } from '@angular/core';
import { ReceivingPercentageWiseService } from '../receiving-percentage-wise.service';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { __values } from 'tslib';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-receiving-percentage-wise',
  templateUrl: './receiving-percentage-wise.component.html',
  styleUrls: ['./receiving-percentage-wise.component.css'],
})
export class ReceivingPercentageWiseComponent {
  @ViewChild('ReceivingPercentageWiseModal') ReceivingPercentageWiseModal;

  bsModalRef: BsModalRef;
  isLoading: boolean = false; // Loader flag

  type: any = 'add';
  percentage: any;
  countryList: any;
  CountryID: any;
  AgentList: any;
  AgentID: any;
  commissionType: any;
  PayOutCountryID: any;
  Data: any;
  receiveCountryInsert: any;
  SendCurrencyInsert: any;
  CurrencyID: any;
  reciveAgentInsert: any;
  Agent_ID: any;
  modeOfPayment: any;
  totalCommissionTypeInsert: any;
  sendCurrencyType: any = [];
  commissionOn: any;
  selectedCurrency: any = [];
  currency: any;
  modeOfPaymentId: any;
  receiveCurrencyType: any;
  ReceivingPercentageWiseForm = new FormGroup({
    PayOutCountryID: new FormControl('None', Validators.required),
    AgentID: new FormControl('None', Validators.required),
    ID: new FormControl('None', Validators.required),
    PayOutCurrencyID: new FormControl('None', Validators.required),
    ModeOfPaymentID: new FormControl('None', Validators.required),
    StartRange: new FormControl('', Validators.required),
    EndRange: new FormControl('', Validators.required),
    PerOnPrincipalAmount: new FormControl('None', Validators.required),
    PayCommissionType: new FormControl('None', Validators.required),
    MinimumCommission: new FormControl('None', Validators.required),
    MaximumCommission: new FormControl('None', Validators.required),
    CommAmtOf: new FormControl('None', Validators.required),
    PayOutCurrencyType: new FormControl('None', Validators.required),
    UserID: new FormControl('None', Validators.required),
  });

  constructor(
    private recivingPrecentWiseSvc: ReceivingPercentageWiseService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService

  ) {}
  ngOnInit(): void {
    this.getReceivingCountryList();
  }

  getReceivingCountryList() {
    this.recivingPrecentWiseSvc.getReceivingCountryList().subscribe((res) => {
      this.countryList = res;
    });
  }

  onSelectCountry($event) {
    this.PayOutCountryID = $event.target.value;
    this.getAgentList($event.target.value);
  }

  onSelectAgent($event) {
    this.AgentID = $event.target.value;
  }

  getAgentList(CountryID: any) {
    this.recivingPrecentWiseSvc.getAgentList(CountryID).subscribe((res) => {
      this.AgentList = res;
    });
  }

  getAllRecord(AgentID: string, PayOutCountryID: string) {
    if (!PayOutCountryID) {
      alert('Please Select Receive Country.');
      return;
    }
    if (!AgentID) {
      alert('Please Select Agent.');
      return;
    }
  
    this.isLoading = true; // Show loader before making request
  
    this.recivingPrecentWiseSvc
      .getAllRecord(AgentID, PayOutCountryID)
      .subscribe({
        next: (res: any) => {
          this.isLoading = false; // Hide loader on successful response
          if (res?.length === 0) {
            alert('No data found.');
            return;
          }
          this.Data = res;
          // Additional processing if needed
        },
        error: (err) => {
          this.isLoading = false; // Hide loader on error
          console.error('Error fetching data:', err);
          // Handle error as needed
        }
      });
  }
  

  updateRecord(payload) {
    this.recivingPrecentWiseSvc.updateRecord(payload[0]).subscribe((res) => {
      res;
    });
  }

  getReceiveCountryInsert() {
    this.recivingPrecentWiseSvc.getReceiveCountryInsert().subscribe((res) => {
      this.receiveCountryInsert = res;
    });
  }

  getSendCurrencyInsert(CountryID: any, AgentID: any) {
    this.recivingPrecentWiseSvc
      .getSendCurrencyInsert(CountryID, AgentID)
      .subscribe((res) => {
        this.SendCurrencyInsert = res;
      });
  }

  getReciveAgentInsert(CountryID: any, CurrencyID: any) {
    this.recivingPrecentWiseSvc
      .getReciveAgentInsert(CountryID, CurrencyID)
      .subscribe((res) => {
        this.reciveAgentInsert = res;
      });
  }

  getModeOfPayment(CountryID: any, AgentID: any) {
    this.recivingPrecentWiseSvc
      .getModeOfPayment(CountryID, AgentID)
      .subscribe((res) => {
        this.modeOfPayment = res;
        console.log('this.modeOfPayment: ', this.modeOfPayment);
      });
  }

  areArraysEqual(sendCurrencyType, currency) {
    if (sendCurrencyType.length !== currency.length) {
      console.log('sendCurrencyType.length: ', sendCurrencyType.length);
      return false;
    }
    sendCurrencyType.sort();
    currency.sort();
    for (let i = 0; i < sendCurrencyType.length; i++) {
      for (let key in sendCurrencyType[i]) {
        if (sendCurrencyType[i][key] !== currency[i][key]) {
          return false;
        }
      }
    }
    return true;
  }

  getSendCurrencyType() {
    this.recivingPrecentWiseSvc.getSendCurrencyType().subscribe((res) => {
      this.sendCurrencyType = res;
      if (!this.areArraysEqual(this.sendCurrencyType, this.currency)) {
        this.sendCurrencyType = this.sendCurrencyType.concat(this.currency);
      }
    });
  }

  getTotalCommissionTypeInsert() {
    this.recivingPrecentWiseSvc
      .getTotalCommissionTypeInsert()
      .subscribe((res) => {
        this.totalCommissionTypeInsert = res;
      });
  }

  getCommissionOn() {
    this.recivingPrecentWiseSvc.getCommissionOn().subscribe((res) => {
      this.commissionOn = res;
    });
  }

  edit(ID: any, type = 'edit') {
    this.type = type;
    this.recivingPrecentWiseSvc.edit(ID).subscribe((res) => {
      this.ReceivingPercentageWiseForm.patchValue({
        ID: res[0].ID,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        PayOutCountryID: res[0].PayOutCountryID,
      });

      this.getSendCurrencyInsert(this.f.PayOutCountryID, this.Agent_ID);

      this.ReceivingPercentageWiseForm.patchValue({
        PayOutCurrencyID: res[0].PayOutCurrencyID,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        AgentID: res[0].Agent_ID,
      });

      this.getReciveAgentInsert(
        this.f.PayOutCountryID,
        this.f.PayOutCurrencyID
      );

      this.ReceivingPercentageWiseForm.patchValue({
        ModeOfPaymentID: res[0].ModeOfPayment_ID,
      });

      this.getModeOfPayment(this.f.PayOutCountryID, this.f.AgentID);

      this.ReceivingPercentageWiseForm.patchValue({
        StartRange: res[0].StartRange,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        EndRange: res[0].EndRange,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        PerOnPrincipalAmount: res[0].PerOnPrincipalAmount,
      });

      this.getCommissionOn();

      this.ReceivingPercentageWiseForm.patchValue({
        PayCommissionType: res[0].PayCommissionType,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        MinimumCommission: res[0].MinCommissionAmount,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        MaximumCommission: res[0].MaxCommissionAmount,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        CommAmtOf: res[0].CommAmtOf,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        PayOutCurrencyType: res[0].PayOutCurrencyType,
      });

      this.ReceivingPercentageWiseForm.patchValue({
        UserID: '0',
      });

      this.onEditReceiveCurrencyType(this.f.PayOutCurrencyID, 'edit');
    });
  }

  get f() {
    return this.ReceivingPercentageWiseForm.value;
  }

  insert(type) {
    if (this.type == 'add') {
      const payload = {
        ID: 0,
        PayOutCountryID: this.CountryID,
        PayOutCurrencyID: this.CurrencyID,
        PayOutCurrencyType: this.receiveCurrencyType,
        AgentID: this.Agent_ID,
        ModeOfPaymentID: this.modeOfPaymentId,
        StartRange: this.f.StartRange,
        EndRange: this.f.EndRange,
        Percentage: this.f.PerOnPrincipalAmount,
        PayCommissionType: this.commissionType,
        MinimumCommission: this.f.MinimumCommission,
        MaximumCommission: this.f.MaximumCommission,
        CommAmtOf: this.f.CommAmtOf,
        UserID: '1',
      };

      if (this.f.PayOutCountryID == 'None') {
        alert('Please Select Receive Country!');
        return;
      }

      if (this.f.StartRange == '') {
        alert('Please Enter Start Range Amount!');
        return;
      }

      if (this.f.EndRange == '') {
        alert('Please Enter End Range Amount!');
        return;
      }
      if (Number(this.f.EndRange) < Number(this.f.StartRange)) {
        alert('End Range should be greater than Start Range!');
        return;
      }
      if (this.f.MinimumCommission == 'None') {
        alert('Please Enter Minimum Commission Amount!');
        return;
      }

      if (this.f.MaximumCommission == 'None') {
        alert('Please Enter Maximum Commission Amount!');
        return;
      }

      this.recivingPrecentWiseSvc.insert(payload).subscribe((res) => {
        res;
        this.closeModal();
      });
    }

    if (this.type == 'edit') {
      if (this.f.PayOutCountryID == 'None') {
        alert('Please Select Receive Country!');
        return;
      }

      if (this.f.StartRange == '') {
        alert('Please Enter Start Range Amount!!');
        return;
      }

      if (this.f.EndRange == '') {
        alert('Please Enter End Range Amount!');
        return;
      }
      if (this.f.EndRange < this.f.StartRange) {
        alert('End Range should be greater than Start Range!');
        return;
      }
      if (this.f.PayOutCurrencyID == 'None') {
        alert('Please Enter Receive Currency Amount!');
        return;
      }
      if (this.f.MinimumCommission == '') {
        alert('Please Enter Minimum Commission Amount!');
        return;
      }

      if (this.f.MaximumCommission == '') {
        alert('Please Enter Maximum Commission Amount!');
        return;
      }

      const payload = this.f;
      console.log('this.f: ', this.f);
      this.recivingPrecentWiseSvc.updateRecord(payload).subscribe((res) => {
        res;
      });
      this.getAllRecord(this.f.AgentID, this.f.PayOutCountryID);
      alert('Record Added successfully!');    
        this.closeModal();
    }
  }

  update() {
    const payload = this.f;
    console.log('this.f: ', this.f);
    this.recivingPrecentWiseSvc.updateRecord(payload).subscribe((res) => {
      res;
    });
    this.getAllRecord(this.f.AgentID, this.f.PayOutCountryID);
    alert('Record updated successfully!');
    this.closeModal();
  }

  onAgentSelect($event) {
    this.Agent_ID = $event.target.value;
    this.getModeOfPayment(this.CountryID, this.Agent_ID);
  }

  onEditReceiveCurrencyType(CurrencyID, type = 'edit') {
    this.getSendCurrencyInsert(this.f.PayOutCountryID, this.f.AgentID);
    this.getReciveAgentInsert(this.f.PayOutCountryID, this.f.PayOutCurrencyID);
    this.getSendCurrencyType();
    this.selectedCurrency = this.SendCurrencyInsert.find(
      (currency) => currency.Currency_Id === parseInt(CurrencyID)
    );

    this.currency = Array(this.selectedCurrency).map((currency) => {
      return {
        ID: currency.Currency_Id,
        CurrencyType: currency.CurrencyShortName,
        // Add more key renaming as needed
      };
    });
    if (type == 'edit') {
      if (!this.areArraysEqual(this.sendCurrencyType, this.currency)) {
        this.sendCurrencyType = this.sendCurrencyType.concat(this.currency);
      }
      this.openReceivingPercentageWiseModal('edit');
    }
  }

  onSelectCurrency($event) {
    this.getReciveAgentInsert(this.CountryID, $event.target.value);
    this.getSendCurrencyType();
    this.CurrencyID = $event.target.value;
    this.selectedCurrency = this.SendCurrencyInsert.find(
      (currency) => currency.Currency_Id === parseInt(this.CurrencyID)
    );

    this.currency = Array(this.selectedCurrency).map((currency) => {
      return {
        ID: currency.Currency_Id,
        CurrencyType: currency.CurrencyShortName,
        // Add more key renaming as needed
      };
    });
  }
  onSelectModeOfPayment($event) {
    this.modeOfPaymentId = $event.target.value;
  }

  onSelectTotalCommissionTypeInsert($event) {
    this.commissionType = $event.target.value;
    if ($event.target.value === '2') {
      this.getCommissionOn();
    } else {
      this.commissionOn = [
        {
          id: 0,
          CommissionAmountOf: 'None',
        },
      ];
    }
  }

  onSelectReceiveCurrencyType($event) {
    this.receiveCurrencyType = $event.target.value;
  }

  SelectCountry($event) {
    this.CountryID = $event.target.value;
    this.getSendCurrencyInsert(this.CountryID, this.Agent_ID);
  }

  onCommissionType($event) {
    this.percentage = $event.target.value;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);

    this.getTotalCommissionTypeInsert();
  }

  closeModal() {
    this.bsModalRef.hide();
  }

  get checkType() {
    let type = this.type;
    switch (type) {
      case 'add':
        return (type = 'Add');
      case 'edit':
        return (type = 'Edit');
      default:
        return type;
    }
  }

  openReceivingPercentageWiseModal(type: any) {
    this.type = type;
    if (type === 'add') {
      this.ReceivingPercentageWiseForm.reset({
        PayOutCountryID: 'None',
        AgentID: 'None',
        PayOutCurrencyID: 'None',
        ModeOfPaymentID: 'None',
        EndRange: '',
        PerOnPrincipalAmount: '',
        PayCommissionType: 'None',
        MinimumCommission: '',
        MaximumCommission: '',
        CommAmtOf: 'None',
        PayOutCurrencyType: '',
        UserID: '',
      });
      this.getReceiveCountryInsert();
      this.openModal(this.ReceivingPercentageWiseModal);
    } else {
      this.getReceiveCountryInsert();
      this.openModal(this.ReceivingPercentageWiseModal);
    }
  }

  get validUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 6 && gUserGroupNumber === 1;
    }

    return false;
  }

  get validAuthorizeUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 1 && gUserGroupNumber === 1;
    }

    return false;
  }
  gridUpdate() {
    this.recivingPrecentWiseSvc
      .gridUpdate(this.GupdateList())
      .subscribe((res) => {
        res;
      });
  }

  GupdateList() {
    const GupdateList = this.Data.filter((con) => con.isChecked).map((con) => ({
      ID: con?.ID,
      StartRange: Number(con?.StartRange),
      EndRange: Number(con?.EndRange),
      Percentage: con?.Percentage,
      MinimumCommission: con?.MinimumCommission,
      MaximumCommission: con?.MaximumCommission,
      UpdatedStartRange: con?.UpdatedStartRange,
      UserID: 0,
      UpdatedEndRange: Number(con?.UpdatedEndRange),
      UpdatedPercentage: Number(con?.UpdatedPercentage),
      UpdatedMinimumCommission: Number(con?.UpdatedMinimumCommission),
      UpdatedMaximumCommission: Number(con?.UpdatedMaximumCommission),
    }));
    return {
      IsValidUser: this.validUser,
      IsValidAuthorizeUser: this.validAuthorizeUser,
      GupdateList: GupdateList,
    };
  }

  Reset() {
    this.ReceivingPercentageWiseForm.reset({
      PayOutCountryID: 'None',
      AgentID: 'None',
      PayOutCurrencyID: 'None',
      ModeOfPaymentID: 'None',
      EndRange: '',
      PerOnPrincipalAmount: '',
      PayCommissionType: 'None',
      MinimumCommission: '',
      MaximumCommission: '',
      CommAmtOf: 'None',
      PayOutCurrencyType: '',
      UserID: '',
    });
  }
}
