import { Component, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { RepushTransactionsService } from '../services/repush-transactions.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-repush-transactions-details',
  templateUrl: './repush-transactions-details.component.html',
  styleUrls: ['./repush-transactions-details.component.css']
})
export class RepushTransactionsDetailsComponent implements OnInit {
  repushTransactionDetails: any[] = [];
  repushTransactionReport: any[] = [];
  showTable: boolean = false;
  form: FormGroup;
  isLoading: boolean = false;
  selectedFileFormat: any = '1';
  @ViewChild('dummyTransactionReceipt') dummyTransactionReceiptModal;
  bsDummyTransactionReceiptModalRef: BsModalRef;

  constructor(
    private fb: FormBuilder,
    private repushTransactionService: RepushTransactionsService,
    private modalService: BsModalService
  ) {
    // Initialize the form with empty values
    this.form = this.fb.group({
      startDate: ['', Validators.required],
      endDate: ['', Validators.required],
    });
  }

  ngOnInit(): void {
    this.setInitialDates();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  setInitialDates(): void {
    const saudiTimeZone = 'Asia/Riyadh';
    const currentDateInSaudi = new Date().toLocaleDateString('en-CA', {
      timeZone: saudiTimeZone
    });

    this.form.patchValue({
      startDate: currentDateInSaudi,
      endDate: currentDateInSaudi,
    });
  }

  fetchData(): void {
    const { startDate, endDate } = this.form.value;
    this.isLoading = true; // Start loading
    this.repushTransactionService.getRepushTransactionReport(startDate, endDate).subscribe(
      (data: any[]) => {
        this.repushTransactionDetails = data;
        this.showTable = true;
        this.isLoading = false; // Stop loading
      },
      (error: any) => {
        console.error('Error fetching instant cash details:', error);
        this.isLoading = false; // Stop loading
      }
    );
  }

  onSubmit(): void {
    if (this.form.valid) {
      this.fetchData();
    } else {
      this.form.markAllAsTouched();
    }
  }

  exportToExcel(): void {
    if (this.repushTransactionDetails && this.repushTransactionDetails.length > 0) {
      /* generate worksheet */
      const ws: XLSX.WorkSheet = XLSX.utils.json_to_sheet(this.repushTransactionDetails);

      /* generate workbook and add the worksheet */
      const wb: XLSX.WorkBook = XLSX.utils.book_new();
      XLSX.utils.book_append_sheet(wb, ws, 'RepushTransactions');

      /* save to file */
      const fileName = `Repush_Transactions_${new Date().getTime()}.xlsx`;
      XLSX.writeFile(wb, fileName);
    } else {
      console.error('No data available to export');
    }
  }

  openDummyTransactionReceipt(transactionID) {
    this.bsDummyTransactionReceiptModalRef = this.modalService.show(this.dummyTransactionReceiptModal, {
      class: 'modal-lg'
    });

    this.isLoading = true; // Start loading
    this.repushTransactionService.Transactionreport(transactionID).subscribe(
      (data: any[]) => {
        this.repushTransactionReport = data[0];
        this.isLoading = false; // Stop loading
      },
      (error: any) => {
        console.error('Error fetching instant cash details:', error);
        this.isLoading = false; // Stop loading
      }
    );

  }

  get startDate() {
    return this.form.get('startDate');
  }

  get endDate() {
    return this.form.get('endDate');
  }
}
