import { Component, ElementRef, TemplateRef, ViewChild } from '@angular/core';
import { SearchTransactionsService } from '../services/search-transactions.service';
import * as XLSX from 'xlsx';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';

@Component({
  selector: 'app-search-transactions',
  templateUrl: './search-transactions.component.html',
  styleUrls: ['./search-transactions.component.css'],
})
export class SearchTransactionsComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;

  lcCurrencyList: any = [];
  selectedLcCurrency: any;
  sendAgentList: any = [];
  selectedSendAgent: any;
  fcCurrencyList: any = [];
  selectedFcCurrency: any;
  receiveAgentList: any = [];
  selectedReceiveAgent: any;
  customerFullName: any = '';
  beneficiaryFullName: any = '';
  fromAmount: any = '';
  toAmount: any = '';
  isUsd: any = false;
  startDate: any = '';
  endDate: any = '';
  allTransactionDetails: any = [];
  reportTypes: any = [
    {
      id: 1,
      type: 'Normal',
    },
    {
      id: 2,
      type: 'CCY Wise',
    },
  ];
  selectedReportType: any = 1;
  isLoading: boolean = false;
  receiptData: any = {};

  constructor(
    private searchTransactionSvc: SearchTransactionsService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getLcCurrencyList();
    this.getFcCurrencyList();
    this.getReceivingAgentList(-1);
    const today = new Date().toISOString().split('T')[0]; // Get today's date in YYYY-MM-DD format
    this.startDate = today;
    this.endDate = today; // Initialize the date model properties with today's date
  }

  getLcCurrencyList() {
    this.searchTransactionSvc.getLcCurrencyList().subscribe({
      next: (res: any) => {
        this.lcCurrencyList = res;
        this.selectedLcCurrency = res?.[0]?.Currency_Id;
        this.getSendAgentList(this.selectedLcCurrency);
      },
      error: () => {
        this.lcCurrencyList = [];
      },
      complete: () => {},
    });
  }

  onLcCurrencyChange(currencyId: any) {
    if (currencyId != -1) {
      this.getSendAgentList(currencyId);
    }
  }

  getSendAgentList(currencyId: any = 0) {
    this.searchTransactionSvc.getSendAgentList(currencyId).subscribe({
      next: (res: any) => {
        this.sendAgentList = res;
        if (currencyId > 0) {
          this.selectedSendAgent = res?.[0]?.agentid;
        }
      },
      error: () => {
        this.sendAgentList = [];
      },
      complete: () => {},
    });
  }

  getFcCurrencyList() {
    this.searchTransactionSvc.getFcCurrencyList().subscribe({
      next: (res: any) => {
        this.fcCurrencyList = res;
        this.selectedFcCurrency = res?.[0]?.Currency_Id;
      },
      error: () => {
        this.fcCurrencyList = [];
      },
      complete: () => {},
    });
  }

  onFcCurrencyChange(currencyId: any) {
    if (currencyId != -1) {
      this.getReceivingAgentList(currencyId);
    }
  }

  getReceivingAgentList(currencyId: any = -1) {
    this.searchTransactionSvc.getReceivingAgentList(currencyId).subscribe({
      next: (res: any) => {
        this.receiveAgentList = res;
        if (currencyId == -1) {
          this.selectedReceiveAgent = res?.[0]?.agentid;
        }
      },
      error: () => {
        this.receiveAgentList = [];
      },
      complete: () => {},
    });
  }

  searchTransaction() {
    if (!this.startDate || !this.endDate) {
      alert('Please Select Start & End Date!');
      return;
    }
    this.isLoading = true;
    let payload: any = {};
    payload = {
      ReportType: Number(this.selectedReportType),
      LcCurrencyId: this.selectedLcCurrency,
      SendAgentId: this.selectedSendAgent ? this.selectedSendAgent : -1,
      FcCurrencyId: this.selectedFcCurrency,
      ReceiveAgentId: this.selectedReceiveAgent
        ? this.selectedReceiveAgent
        : -1,
      CustFullname: this.customerFullName ? this.customerFullName : '',
      Benfullname: this.beneficiaryFullName ? this.beneficiaryFullName : '',
      Fromamount: this.fromAmount ? this.fromAmount : '',
      ToAmount: this.toAmount ? this.toAmount : '',
      // IsAmountinUSD: this.isUsd ? '1' : '0',
      IsAmountinUSD: this.isUsd,
      StartDate: this.startDate,
      EndDate: this.endDate,
    };
    this.searchTransactionSvc.searchTransactionDetail(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          this.isLoading = false;
          alert('No data found.');
          return;
        }
        this.allTransactionDetails = res.map((element: any) => {
          element.PayInAmount = this.formatWithTwoDecimalPlaces(
            element.PayInAmount
          );
          element.ExchGain = this.formatWithTwoDecimalPlaces(element.ExchGain);
          element.DebitAmount = this.formatWithTwoDecimalPlaces(
            element.DebitAmount
          );
          element.DebitUSD = this.formatWithTwoDecimalPlaces(element.DebitUSD);
          element.Comm = this.formatWithTwoDecimalPlaces(element.Comm);
          element.VATAmount = this.formatWithTwoDecimalPlaces(
            element.VATAmount
          );
          element.PayOutAmount = this.formatWithTwoDecimalPlaces(
            element.PayOutAmount
          );
          element.RASettelmentRate = this.formatWithFourDecimalPlaces(
            element.RASettelmentRate
          );
          return element;
        });
      },
      error: () => {
        this.allTransactionDetails = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Search_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  getTotal(property: string): number {
    const sanitizedProperty = property.replace(/\s+/g, '');

    if (this.allTransactionDetails?.[0]?.hasOwnProperty(sanitizedProperty)) {
      return this.allTransactionDetails[0][sanitizedProperty];
    }

    return 0;
  }

  formatWithTwoDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 2 decimal places
      return `${integerPart}.${fractionalPart.padEnd(2, '0').slice(0, 2)}`;
    } else {
      // If no decimal places, add two zeros
      return `${integerPart}.00`;
    }
  }

  formatWithFourDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 4 decimal places
      return `${integerPart}.${fractionalPart.padEnd(4, '0').slice(0, 4)}`;
    } else {
      // If no decimal places, add four zeros
      return `${integerPart}.0000`;
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.TransactionNumber;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }
}
