import { Component, TemplateRef, ViewChild } from '@angular/core';
import { SendAgentsCtaTransactionsService } from '../send-agents-cta-transactions.service';
import { DatePipe } from '@angular/common';
import { AuthService } from '../services/auth.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';
import { getCacheData } from '../utils/helper';
@Component({
  selector: 'app-send-agents-cta-transactions',
  templateUrl: './send-agents-cta-transactions.component.html',
  styleUrls: ['./send-agents-cta-transactions.component.css'],
})
export class SendAgentsCtaTransactionsComponent {
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;

  constructor(
    private SendAgentsCtaTransactionsSvc: SendAgentsCtaTransactionsService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private datePipe: DatePipe,
    private authSvc: AuthService,
    private modalService: BsModalService
  ) {}

  AgentID: any;
  CountryList: any;
  toDate: any = Date.now();
  AgentList: any;
  allData: any = [];
  isLoading: boolean = false;
  receiptData: any = {};

  ngOnInit(): void {
    this.getCountry();
    this.getAgent();
    this.toDate = this.authSvc.getTodayDate();
  }

  onSelectDate() {
    this.getCountry();
    this.getAgent();
  }

  getCountry() {
    const date = this.datePipe.transform(this.toDate, 'yyyy-MM-dd');
    this.SendAgentsCtaTransactionsSvc.getCountry(date).subscribe((res) => {
      this.CountryList = res;
    });
  }

  getAgent() {
    const date = this.datePipe.transform(this.toDate, 'yyyy-MM-dd');
    this.SendAgentsCtaTransactionsSvc.getAgent(date).subscribe((res) => {
      this.AgentList = res;
      this.AgentID = res[0].Agent_ID;
    });
  }

  onSelectAgent(AgentID) {
    this.AgentID = AgentID;
  }

  allRecord() {
    this.isLoading = true;

    const date = this.datePipe.transform(this.toDate, 'yyyy-MM-dd');
    this.SendAgentsCtaTransactionsSvc.getAllRecord(
      date,
      this.AgentID
    ).subscribe((res: any) => {
      if (res?.length == 0) {
        alert('No data found.');
        return;
      }
      this.allData = res;
      this.isLoading = false;
    });
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.SRTRNo;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }

  selectAll(event: any) {
    const checked = event.target.checked;
    this.allData.forEach((record) => (record.selected = checked));
  }

  releaseTransaction() {
    const selectedRecords = this.allData.filter((record) => record.selected);
    if (selectedRecords?.length == 0) {
      alert('Please Select Transaction for Release.');
      return;
    }
    const payload = selectedRecords.map((record) => ({
      IsChk: true,
      TransactionNumber: record.SRTRNo,
      G_UserID: getCacheData('G_UserID'),
    }));
    this.SendAgentsCtaTransactionsSvc.releaseTransaction(payload).subscribe({
      next: (res: any) => {
        if (res) {
          alert('Transactions has been released.');
          this.allRecord();
        }
      },
      error: (err) => {
        alert(err?.error?.message);
      },
      complete: () => {},
    });
  }
}
