import { Component, TemplateRef, ViewChild } from '@angular/core';
import { SendingPercentageWiseService } from '../services/sending-percentage-wise.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { CountryWiseCommissionService } from '../services/country-wise-commission.service';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-sending-percentage-wise',
  templateUrl: './sending-percentage-wise.component.html',
  styleUrls: ['./sending-percentage-wise.component.css'],
})
export class SendingPercentageWiseComponent {
  @ViewChild('sendingPercentage') sendingPercentageModal;
  bsModalRef: BsModalRef;
  isLoading: boolean = false; // Loader flag

  countryList: any = [];
  selectedCountry: any = '0';
  agentList: any = [];
  selectedAgent: any = '0';

  sendingPercentageWise: any = [];
  type: any = 'add';

  countryInsertList: any = [];
  sendCurrencyList: any = [];
  sendAgentInsertList: any = [];
  receiveCountryList: any = [];
  receiveCurrencyList: any = [];
  agentAccountList: any = [];
  modeofPaymentList: any = [];
  commissionTypeList: any = [];

  endRangeError: boolean = false;

  sendingPercentageForm = new FormGroup({
    ID: new FormControl(),
    PayInCountryID: new FormControl('None', Validators.required),
    PayInCurrencyID: new FormControl('None', Validators.required),
    AgentID: new FormControl('None', Validators.required),
    PayOutCountryID: new FormControl('None', Validators.required),
    PayOutCurrencyID: new FormControl('None', Validators.required),
    ToAgentID: new FormControl('None', Validators.required),
    ModeOfPaymentID: new FormControl('None', Validators.required),
    StartRange: new FormControl('', Validators.required),
    EndRange: new FormControl('', Validators.required),
    Percentage: new FormControl('', Validators.required),
    PayCommissionType: new FormControl('', Validators.required),
    MinimumCommission: new FormControl('', Validators.required),
    MaximumCommission: new FormControl('', Validators.required),
  });

  constructor(
    private sendingPercentageService: SendingPercentageWiseService,
    private countryWiseService: CountryWiseCommissionService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService

  ) { }

  ngOnInit(): void {
    this.getSendCountryList();
    this.getInsertSendCountry();
    this.getTotalCommissionType();
  }

  getSendCountryList() {
    this.sendingPercentageService.getSendCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => { },
    });
  }

  onCountryChange(countryId: any) {
    if (countryId) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sendingPercentageService.getSendAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => { },
    });
  }

  filter() {
    this.isLoading = true; // Show loader before making request
  
    this.sendingPercentageService
      .getFillGrid(this.selectedCountry, this.selectedAgent)
      .subscribe({
        next: (res: any) => {
          this.isLoading = false; // Hide loader on successful response
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.sendingPercentageWise = res;
          this.formatSendingPercentageWise(this.sendingPercentageWise);
        },
        error: (err) => {
          this.isLoading = false; // Hide loader on error
          this.sendingPercentageWise = [];
        },
        complete: () => { },
      });
  }
  

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, {
      class: 'modal-lg'
    });
  }

  openSendingPercentageWiseModal(type: any) {
    if (type == 'add') {
      this.sendingPercentageForm.reset({
        PayInCountryID: 'None',
      });
    }
    this.type = type;
    this.openModal(this.sendingPercentageModal);
  }

  get f() {
    return this.sendingPercentageForm.value;
  }

  getInsertSendCountry() {
    this.countryWiseService
      .getInsertSendCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.countryInsertList = res;
        },
        error: (err) => {
          this.countryInsertList = [];
        },
        complete: () => { },
      });
  }

  onCountryInsertChange(countryId) {
    if (countryId != 'None') {
      this.getInsertSendCurrency(countryId);
      this.getReceiveCountryList();
    }
  }

  getInsertSendCurrency(payInCountryId: any = 0, currencyId = 0) {
    this.countryWiseService
      .getInsertSendCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendCurrencyList = res;
          if (this.type === 'add') {
            this.setPayInCurrencyID('None');
          } else {
            const payInCurrencyId = this.findPayInCurrencyId(currencyId);
            this.setPayInCurrencyID(payInCurrencyId);
          }
        },
        error: (err) => {
          this.sendCurrencyList = [];
        },
        complete: () => { },
      });
  }

  onSendCurrencyChange(currencyId) {
    if (currencyId != 'None' && this.f.PayInCountryID != 'None') {
      this.getSendAgentInsertList(currencyId, this.f.PayInCountryID);
    }
  }

  getSendAgentInsertList(currencyId: any = 0, countryId: any = 0, agentId = 0) {
    this.sendingPercentageService
      .getSendAgentInsertList(currencyId, countryId)
      .subscribe({
        next: (res: any) => {
          this.sendAgentInsertList = res;
          if (this.type === 'add') {
            this.setAgentID('None');
          } else {
            const agentID = this.findAgentID(agentId);
            this.setAgentID(agentID);
          }
        },
        error: (err) => {
          this.sendAgentInsertList = [];
        },
        complete: () => { },
      });
  }

  getReceiveCountryList(countryId: any = 0, receiveCountry = 0) {
    this.sendingPercentageService.getReceiveCountryList(countryId).subscribe({
      next: (res: any) => {
        this.receiveCountryList = res;
        if (this.type === 'add') {
          this.setPayOutCountryID('None');
        } else {
          const receiveCountryId = this.findReceiveCountryID(receiveCountry);
          this.setPayOutCountryID(receiveCountryId);
        }
      },
      error: (err) => {
        this.receiveCountryList = [];
      },
      complete: () => { },
    });
  }

  onReceiveCountryChange(countryId) {
    if (countryId != 'None') {
      this.getReceiveCurrencyList(countryId);
    }
  }

  getReceiveCurrencyList(countryId: any = 0, receiveCurrency = 0) {
    this.sendingPercentageService.getReceiveCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.receiveCurrencyList = res;
        if (this.type === 'add') {
          this.setPayOutCurrencyID('None');
        } else {
          const receiveCurrencyId = this.findReceiveCurrencyID(receiveCurrency);
          this.setPayOutCurrencyID(receiveCurrencyId);
        }
      },
      error: (err) => {
        this.receiveCurrencyList = [];
      },
      complete: () => { },
    });
  }

  onReceiveCurrencyChange(currencyId) {
    if (currencyId != 'None' && this.f.PayOutCountryID != 'None') {
      this.getAgentAccountList(this.f.PayOutCountryID, currencyId);
    }
  }

  getAgentAccountList(countryId: any = 0, currencyId: any = 0, receiveAgent = 0) {
    this.sendingPercentageService
      .getAgentAccountList(countryId, currencyId)
      .subscribe({
        next: (res: any) => {
          this.agentAccountList = res;
          if (this.type === 'add') {
            this.setToAgentID('None');
          } else {
            const receiveAgentId = this.findToAgentID(receiveAgent);
            this.setToAgentID(receiveAgentId);
          }
        },
        error: (err) => {
          this.agentAccountList = [];
        },
        complete: () => { },
      });
  }

  onReceiveAgentChange(agentId) {
    if (agentId != 'None' && this.f.PayOutCountryID != 'None') {
      this.getModeofPaymentList(this.f.PayOutCountryID, agentId);
    }
  }

  getModeofPaymentList(countryId: any = 0, receiveAgentId: any = 0, modeofPayment = 0) {
    this.sendingPercentageService
      .getModeofPaymentList(countryId, receiveAgentId)
      .subscribe({
        next: (res: any) => {
          this.modeofPaymentList = res;
          if (this.type === 'add') {
            this.setModeOfPaymentID('None');
          } else {
            const modeofPaymentId = this.findModeOfPaymentID(modeofPayment);
            this.setModeOfPaymentID(modeofPaymentId);
            this.openSendingPercentageWiseModal('edit');
          }
        },
        error: (err) => {
          this.modeofPaymentList = [];
        },
        complete: () => { },
      });
  }

  getTotalCommissionType() {
    this.countryWiseService
      .getTotalCommissionType(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.commissionTypeList = res;
          this.sendingPercentageForm.patchValue({
            PayCommissionType: 'None',
          });
        },
        error: (err) => {
          this.commissionTypeList = [];
        },
        complete: () => { },
      });
  }

  allowOneDecimal(event: Event, range: string): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;

    this.checkEndRange(range);
  }

  checkEndRange(rangeType: string): void {
    const startRange = this.sendingPercentageForm.get('StartRange').value;
    const endRange = this.sendingPercentageForm.get('EndRange').value;

    switch (rangeType) {
      case 'EndRange':
        this.endRangeError = Number(endRange) < Number(startRange);
        break;

      default:
        break;
    }
  }

  addSendingPercentage() {
    if (!this.sendingPercentageForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (
      !this.checkSelection(this.f.PayInCountryID, 'Please Select Send Country.')
    ) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.PayInCurrencyID,
        'Please Select Send Currency.'
      )
    ) {
      return;
    }

    if (!this.checkSelection(this.f.AgentID, 'Please Select Sending Agent.')) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.PayOutCountryID,
        'Please Select Receive Country.'
      )
    ) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.PayOutCurrencyID,
        'Please Select Receive Currency.'
      )
    ) {
      return;
    }

    if (
      !this.checkSelection(this.f.ToAgentID, 'Please Select Receiving Agent.')
    ) {
      return;
    }

    if (
      !this.checkSelection(
        this.f.ModeOfPaymentID,
        'Please Select Mode of Payment.'
      )
    ) {
      return;
    }

    let payload;
    payload = {
      ID: 0,
      PayInCountryID: this.f.PayInCountryID,
      PayInCurrencyID: this.f.PayInCurrencyID,
      AgentID: this.f.AgentID,
      PayOutCountryID: this.f.PayOutCountryID,
      PayOutCurrencyID: this.f.PayOutCurrencyID,
      ToAgentID: this.f.ToAgentID,
      ModeOfPaymentID: this.f.ModeOfPaymentID,
      StartRange: this.f.StartRange,
      EndRange: this.f.EndRange,
      Percentage: this.f.Percentage,
      PayCommissionType: this.f.PayCommissionType,
      MinimumCommission: this.f.MinimumCommission,
      MaximumCommission: this.f.MaximumCommission,
      EnteredBy: getCacheData('G_UserID'),
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: Number(this.f.ID)
      };
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addSendingPercentageWise'
        : 'updateSendingPercentageWise';

    this.sendingPercentageService[serviceMethod](payload).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        const message = this.type === 'edit' ? 'updated' : 'added';
        alert(`Sending Percentage Wise information ${message} successfully.`);
      },
      error: (err) => {
        this.modalService.hide();
        // alert(err?.error?.message || 'Something went wrong');
        this.toastrSvc.error(err?.error?.message)
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  checkSelection(value: string, errorMessage: string): boolean {
    if (value == 'None') {
      alert(errorMessage);
      return false;
    }
    return true;
  }

  formatSendingPercentageWise(array: any = []) {
    for (let index = 0; index < array.length; index++) {
      const element = array[index];
      if (element?.StartRange !== null && element?.StartRange !== undefined) {
        element.StartRange = this.formatWithThreeDecimalPlaces(
          element.StartRange
        );
      }
      if (element?.EndRange !== null && element?.EndRange !== undefined) {
        element.EndRange = this.formatWithThreeDecimalPlaces(element.EndRange);
      }
      if (element?.Percentage !== null && element?.Percentage !== undefined) {
        element.Percentage = this.formatWithThreeDecimalPlaces(
          element.Percentage
        );
      }
      if (
        element?.MinimumCommission !== null &&
        element?.MinimumCommission !== undefined
      ) {
        element.MinimumCommission = this.formatWithThreeDecimalPlaces(
          element.MinimumCommission
        );
      }
      if (
        element?.MaximumCommission !== null &&
        element?.MaximumCommission !== undefined
      ) {
        element.MaximumCommission = this.formatWithThreeDecimalPlaces(
          element.MaximumCommission
        );
      }
    }
  }

  formatWithThreeDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 3 decimal places
      return `${integerPart}.${fractionalPart.padEnd(3, '0').slice(0, 3)}`;
    } else {
      // If no decimal places, add 3 zeros
      return `${integerPart}.000`;
    }
  }

  reset() {
    this.sendingPercentageForm.reset({
      PayInCountryID: 'None',
    });
  }

  edit(Id: any) {
    this.sendingPercentageService.editSendingPercentageWise(Id).subscribe({
      next: (res: any) => {
        this.mappedSendingPercentage(res?.[0]);
      },
      error: (err) => {
        // alert(err?.error?.message || 'Something went wrong');
        this.toastrSvc.error(err?.error?.message)
      },
      complete: () => {
      },
    });
  }

  mappedSendingPercentage(sendingPercentage) {

    console.log(sendingPercentage)
    this.type = 'edit';
    const payInCountryId = this.countryInsertList.find((c) => c.CountryId == sendingPercentage.PayInCountryID)?.CountryId;
    const SendAgent_ID = this.agentList.find((c) => c.Agent_ID == sendingPercentage.Agent_ID)?.Agent_ID;
    this.sendingPercentageForm.patchValue({
      ID: sendingPercentage?.ID,
      PayInCountryID: payInCountryId,
      AgentID: SendAgent_ID,
      StartRange: sendingPercentage?.StartRange,
      EndRange: sendingPercentage?.EndRange,
      Percentage: sendingPercentage?.Percentage,
      PayCommissionType: sendingPercentage?.PayCommissionType,
      MinimumCommission: sendingPercentage?.MinimumCommission,
      MaximumCommission: sendingPercentage?.MaximumCommission
    });
    this.getInsertSendCurrency(payInCountryId, sendingPercentage?.PayInCurrencyID);
    this.getSendAgentInsertList(sendingPercentage?.PayInCurrencyID, payInCountryId, sendingPercentage?.Agent_ID);
    this.getReceiveCountryList(0, sendingPercentage?.PayOutCountryID);
    this.getReceiveCurrencyList(sendingPercentage?.PayOutCountryID, sendingPercentage?.PayOutCurrencyID);
    this.getAgentAccountList(sendingPercentage?.PayOutCountryID, sendingPercentage?.PayOutCurrencyID, sendingPercentage?.ToAgent_ID);
    this.getModeofPaymentList(sendingPercentage?.PayOutCountryID, sendingPercentage?.ToAgent_ID, sendingPercentage?.ModeOfPayment_ID);
  }

  setPayInCurrencyID(payInCurrencyId: any) {
    this.sendingPercentageForm.patchValue({
      PayInCurrencyID: payInCurrencyId,
    });
  }

  findPayInCurrencyId(currencyId: number): any {
    return this.sendCurrencyList.find((c) => c.Currency_Id === currencyId)?.Currency_Id || 'None';
  }

  setAgentID(agentID: any) {
    this.sendingPercentageForm.patchValue({
      AgentID: agentID,
    });
  }

  findAgentID(agentId: number): any {
    return this.sendAgentInsertList.find((a) => a.Agent_ID === agentId)?.Agent_ID || 'None';
  }

  setPayOutCountryID(payOutCountryID: any) {
    this.sendingPercentageForm.patchValue({
      PayOutCountryID: payOutCountryID,
    });
  }

  findReceiveCountryID(receiveCountry: number): any {
    return this.receiveCountryList.find((a) => a.CountryID === receiveCountry)?.CountryID || 'None';
  }

  setPayOutCurrencyID(payOutCurrencyID: any) {
    this.sendingPercentageForm.patchValue({
      PayOutCurrencyID: payOutCurrencyID,
    });
  }

  findReceiveCurrencyID(receiveCurrency: number): any {
    return this.receiveCurrencyList.find((c) => c.Currency_Id === receiveCurrency)?.Currency_Id || 'None';
  }

  setToAgentID(toAgentID: any) {
    this.sendingPercentageForm.patchValue({
      ToAgentID: toAgentID,
    });
  }

  findToAgentID(receiveAgent: number): any {
    return this.agentAccountList.find((a) => a.Agent_ID === receiveAgent)?.Agent_ID || 'None';
  }

  setModeOfPaymentID(modeOfPaymentID: any) {
    this.sendingPercentageForm.patchValue({
      ModeOfPaymentID: modeOfPaymentID,
    });
  }

  findModeOfPaymentID(modeofPayment: number): any {
    return this.modeofPaymentList.find((m) => m.ModeOfPayment_ID === modeofPayment)?.ModeOfPayment_ID || 'None';
  }

  setResponseMessage(message: string): any {
    return alert(message);
  }

  get validUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 6 && gUserGroupNumber === 1;
    }

    return false;
  }

  get validAuthorizeUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 1 && gUserGroupNumber === 1;
    }

    return false;
  }


  gridUpdate() {
    const checkedList = this.sendingPercentageWise.filter((con) => con.isChecked == true);
    if (checkedList?.length) {
      this.sendingPercentageService
        .gridUpdate(this.updateList())
        .subscribe({
          next: (res: any) => {
            this.filter();
            alert('Commission Slab Update Successfully.');
          },
          error: (err) => {
            // alert(err?.error?.message || 'Something went wrong');
            this.toastrSvc.error(err?.error?.message)
          },
          complete: () => {
          },
        });
    }
  }

  updateList() {
    const updateList = this.sendingPercentageWise.filter((con) => con.isChecked == true).map((con) => ({
      ID: con?.ID,
      StartRange: Number(con?.UpdatedStartRange) ? Number(con?.UpdatedStartRange) : Number(con?.StartRange),
      EndRange: Number(con?.UpdatedEndRange) ? Number(con?.UpdatedEndRange) : Number(con?.EndRange),
      Percentage: Number(con?.UpdatedPercentage) ? Number(con?.UpdatedPercentage) : Number(con?.Percentage),
      MinimumCommission: Number(con?.UpdatedMinimumCommission) ? Number(con?.UpdatedMinimumCommission) : Number(con?.MinimumCommission),
      MaximumCommission: Number(con?.UpdatedMaximumCommission) ? Number(con?.UpdatedMaximumCommission) : Number(con?.MaximumCommission),
      UserID: getCacheData('G_UserID'),
      UpdatedStartRange: con?.UpdatedStartRange ? con?.UpdatedStartRange : con?.StartRange,
      UpdatedEndRange: Number(con?.UpdatedEndRange) ? Number(con?.UpdatedEndRange) : Number(con?.EndRange),
      UpdatedPercentage: Number(con?.UpdatedPercentage) ? Number(con?.UpdatedPercentage) : Number(con?.Percentage),
      UpdatedMinimumCommission: Number(con?.UpdatedMinimumCommission) ? Number(con?.UpdatedMinimumCommission) : Number(con?.MinimumCommission),
      UpdatedMaximumCommission: Number(con?.UpdatedMaximumCommission) ? Number(con?.UpdatedMaximumCommission) : Number(con?.MaximumCommission)
    }));
    return {
      IsValidUser: this.validUser,
      IsValidAuthorizeUser: this.validAuthorizeUser,
      GupdateList: updateList,
    };
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
