import { Component, OnInit } from '@angular/core';
import { SendTxnsUserWiseService } from '../services/send-txns-user-wise.service';
import { getCacheData } from '../utils/helper';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-sent-txns-user-wise',
  templateUrl: './sent-txns-user-wise.component.html',
  styleUrls: ['./sent-txns-user-wise.component.css'],
})
export class SentTxnsUserWiseComponent implements OnInit {
  // fromDate: any = '';
  // toDate: any = '';
  fromDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  countries: any[] = [];
  selectedCountry: any;
  agents: any[] = [];
  selectedAgent: any;
  agentBranches: any[] = [];
  selectedAgentBranch: any;
  branchUsers: any[] = [];
  selectedBranchUser: any;
  selectedReportType: any = '0';
  isLoading: boolean = false;
  detailsGrid: any = [];
  summaryGrid: any = [];
  selectedFileFormat: string = 'excel';

  constructor(private sendTxnsUserWiseService: SendTxnsUserWiseService) {}

  ngOnInit(): void {
    this.getSentTransactionsForSalespersonByCountry();
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getSentTransactionsForSalespersonByCountry() {
    this.sendTxnsUserWiseService
      .getSentTransactionsForSalespersonByCountry(
        getCacheData('G_UserID'),
        1,
        0
      )
      .subscribe({
        next: (res: any) => {
          this.countries = res;
          this.selectedCountry = res?.[0]?.CountryID;
          this.getSentTransactionsForSalespersonByAgent(this.selectedCountry);
        },
        error: (err) => {
          this.countries = [];
        },
        complete: () => {},
      });
  }

  onCountryChange(countryId) {
    if (countryId) {
      this.getSentTransactionsForSalespersonByAgent(countryId);
    }
  }

  getSentTransactionsForSalespersonByAgent(country_id = 0) {
    this.sendTxnsUserWiseService
      .getSentTransactionsForSalespersonByAgent(0, country_id)
      .subscribe({
        next: (res: any) => {
          this.agents = res;
          this.selectedAgent = res?.[0]?.Agent_ID;
        },
        error: (err) => {
          this.agents = [];
        },
        complete: () => {},
      });
  }

  onAgentChange(agentId) {
    if (agentId) {
      this.getSentTransactionsForSalespersonByAgentBranch(agentId);
    }
  }

  getSentTransactionsForSalespersonByAgentBranch(agentId = 0, branchId = '-1') {
    this.sendTxnsUserWiseService
      .getSentTransactionsForSalespersonByAgentBranch(agentId, branchId)
      .subscribe({
        next: (res: any) => {
          this.agentBranches = res;
          this.selectedAgentBranch = res?.[0]?.AgentBranch_ID;
          this.getUserListForAgentBranch(this.selectedAgentBranch);
        },
        error: (err) => {
          this.agentBranches = [];
        },
        complete: () => {},
      });
  }

  onAgentBranchChange(agentBranchId) {
    if (agentBranchId) {
      this.getUserListForAgentBranch(agentBranchId);
    }
  }

  getUserListForAgentBranch(branchId = 0) {
    this.sendTxnsUserWiseService.getUserListForAgentBranch(branchId).subscribe({
      next: (res: any) => {
        this.branchUsers = res;
        this.selectedBranchUser = res?.[0]?.UserId;
      },
      error: (err) => {
        this.branchUsers = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From and To Dates.');
      return;
    }
    if (!this.selectedCountry) {
      alert('Please Select Country.');
      return;
    }
    if (!this.selectedAgent) {
      alert('Please Select Agent.');
      return;
    }
    if (!this.selectedAgentBranch) {
      alert('Please Select Agent Branch.');
      return;
    }
    if (!this.selectedBranchUser) {
      alert('Please Select Branch User.');
      return;
    }
    if (!this.selectedReportType) {
      alert('Please Select Report Type.');
      return;
    }
    let agentBranchId;
    if (this.selectedAgentBranch == '-1') {
      agentBranchId = 'ALL';
    }
    this.isLoading = true;
    const payload = {
      CountryID: String(this.selectedCountry),
      AgentID: String(this.selectedAgent),
      AgentBranchID: agentBranchId ? agentBranchId : this.selectedAgentBranch,
      UserID: '-1',
      FromDate: this.fromDate,
      ToDate: this.toDate,
      UserGroupType: '1',
      SessionAgent_ID: '0',
    };
    if (this.selectedReportType == '0') {
      this.summaryGrid = [];
      this.sendTxnsUserWiseService.getDetailGrid(payload).subscribe({
        next: (res: any) => {
          this.isLoading = false;
          if (res?.length == 0) {
            alert('No Data Found.');
            return;
          }
          this.detailsGrid = res;
          this.detailsGrid = res?.flatMap((item) => item.Table);
        },
        error: (err) => {
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
    } else {
      this.detailsGrid = [];
      this.sendTxnsUserWiseService.getDetailSummary(payload).subscribe({
        next: (res: any) => {
          this.isLoading = false;
          if (res?.length == 0) {
            alert('No Data Found.');
            return;
          }
          this.summaryGrid = res;
          this.summaryGrid = res?.flatMap((item) => item.SummaryTable);
          const sanitizedDataArray = this.removeHtmlTagsFromArrayOfObjects(
            this.summaryGrid
          );
          this.summaryGrid = sanitizedDataArray;
        },
        error: (err) => {
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
    }
  }

  removeHtmlTagsFromArrayOfObjects(arr: any[]): any[] {
    return arr.map((obj) => {
      const newObj: any = {};
      for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
          const value = obj[key];
          if (typeof value === 'string') {
            newObj[key] = value.replace(/<[^>]*>/g, ''); // Remove HTML tags using regex
          } else {
            newObj[key] = value;
          }
        }
      }
      return newObj;
    });
  }

  get agentName() {
    let agent = '--ALL--';
    if (this.selectedAgent != '-1') {
      agent = this.agents.find(
        (a) => a.Agent_ID == this.selectedAgent
      )?.Agent_Name;
    }
    return agent;
  }

  exportToExcel() {
    let ws: XLSX.WorkSheet;

    // Generate worksheet based on data availability
    if (this.detailsGrid?.length > 0) {
      ws = XLSX.utils.json_to_sheet(this.detailsGrid);
    } else {
      ws = XLSX.utils.json_to_sheet(this.summaryGrid);
    }

    // Generate workbook and add the worksheet
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    // Determine file extension and save to file
    const fileExtension = this.selectedFileFormat === 'excel' ? 'xlsx' : 'csv';
    const fileName = `Sent_Txns_Report${Date.now()}.${fileExtension}`;
    XLSX.writeFile(wb, fileName);
  }
}
