import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';

@Injectable({
  providedIn: 'root',
})
export class CifWiseMarginsService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getSendingCountry(token: any, username) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins/sending-country`,
      {
        headers: headers,
      }
    );
  }

  getReceivingCountry(
    token: any,
    username,
    countryId: any = 0
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins/receiving-country?countryID=${countryId}
     `,
      {
        headers: headers,
      }
    );
  }

  getAllCIFWiseMargins(
    token: any,
    username,
    payInCountry: any = 0,
    payOutCountry: any = 0
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins?payInCountry=${payInCountry}&payOutCountry=${payOutCountry}
     `,
      {
        headers: headers,
      }
    );
  }

  getSendCurrency(token: any, username, countryId: any = 0): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins/sending-currency?countryID=${countryId}
     `,
      {
        headers: headers,
      }
    );
  }

  getReceiveCurrency(
    token: any,
    username,
    countryId: any = 0
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins/receiving-currency?countryID=${countryId}
     `,
      {
        headers: headers,
      }
    );
  }

  getModeOfPayment(
    token: any,
    username,
    payOutCountryId: any = 0
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins/mode-of-payment?payOutCountryId=${payOutCountryId}
     `,
      {
        headers: headers,
      }
    );
  }

  getCIFNumbers(token: any, username): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins/cif-number-list
     `,
      {
        headers: headers,
      }
    );
  }

  addCIFWiseMargin(token: any, username, payload: any): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.post(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins
     `,
      payload,
      {
        headers: headers,
      }
    );
  }

  updateCIFWiseMargin(token: any, username, payload: any): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.patch(
      `${environment.apiUrl}/api/master/exchange-margin/agency-margin/cif-wise-margins
     `,
      payload,
      {
        headers: headers,
      }
    );
  }
}
