import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class PermissionService {
  private permissionsSubject = new BehaviorSubject<any[]>([]);
  public permissions$ = this.permissionsSubject.asObservable();

  constructor() {}

  updatePermissions(permissions: any[]): void {
    this.permissionsSubject.next(permissions);
  }

  checkPermission(
    firstMenu: string,
    secondMenu?: string,
    thirdMenu?: string
  ): any {
    const permissions = this.permissionsSubject.value;

    // Check if parent menu is visible
    const parentPermission = permissions.find(
      (permission) =>
        permission.FirstMenu === firstMenu && permission.FirstActive === true
    );

    if (!parentPermission) {
      return null;
    }

    if (secondMenu && thirdMenu) {
      // Check if third menu is visible
      const thirdPermission = permissions.find(
        (permission) =>
          permission.FirstMenu === firstMenu &&
          permission.SecondMenu === secondMenu &&
          permission.ThirdMenu === thirdMenu &&
          permission.ThirdActive === true
      );

      return thirdPermission || null;
    }

    if (secondMenu) {
      // Check if second menu is visible
      const secondPermission = permissions.find(
        (permission) =>
          permission.FirstMenu === firstMenu &&
          permission.SecondMenu === secondMenu &&
          permission.SecondActive === true
      );

      return secondPermission || null;
    }

    return parentPermission;
  }
}
