import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class SalesMarketingReportService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getReportTypes(): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agency/sales-marketing-reports/report-type`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendCountries(): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agency/mis-audit-reports/send-country`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendAgents(countryId: number, criteria: string): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agency/sales-marketing-reports/agent?CountryID=${countryId}&Criteria=${criteria}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentBranches(agentId: number): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agency/sales-marketing-reports/agent-branch?agentId=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
