import { Component } from '@angular/core';
import { StopPaymentService } from '../services/stop-payment.service';
import { Router } from '@angular/router';
import { AuthService } from '../services/auth.service';
import { getCacheData } from '../utils/helper';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-stop-payment',
  templateUrl: './stop-payment.component.html',
  styleUrls: ['./stop-payment.component.css'],
})
export class StopPaymentComponent {
  status: any = [];
  selectedStatus: any;
  allStatus: any = [];
  selectedAllStatus: any;
  agentTypes: any = [];
  selectedAgentType: any;
  searchByType: any = 1;
  viewLast: any = 10;
  isLoading: boolean = false;
  selectAll: boolean = false;

  AllData: any = [];
  typeList: any = [
    {
      id: 1,
      name: 'Send',
    },
  ];
  selecteStatus: any;
  selectedType: any = 1;
  sendCountryList: any = [];
  selectedSendCountryList: any = 'All';
  destCountryList: any = [];
  selectedDestCountryList: any = 'All';
  sendCurrencyList: any = [];
  selectedSendCurrencyList: any;
  destCurrencyList: any = [];
  selectedDestCurrencyList: any;
  agentList: any = [];
  selectedAgentList: any = 'All';
  userList: any = [];
  selectedUserList: any = '0';
  agentBranchList: any = [];
  selectedAgentBranchList: any = '0';
  transferTypeList: any = [];
  selectedTransferTypeList: any = 'All';

  startDate: any;
  endDate: any;
  transactionNo: any;
  customerNameFirst: any;
  customerNameLast: any;
  beneficiaryNameFirst: any;
  beneficiaryNameLast: any;

  constructor(
    private stopPaymentSvc: StopPaymentService,
    private router: Router,
    private authSvc: AuthService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit(): void {
    this.getStatusList();
    this.getAllStatusList();
    this.getAgentTypeList();
    this.getSendCountryList();
    this.getDestinationCountryList();
    this.getTransferTypeList();
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  toggleSelectAll() {
    this.AllData.forEach((d: any) => (d.selected = this.selectAll));
  }

  checkIfAllSelected() {
    this.selectAll = this.AllData.every((d: any) => d.selected);
  }
  getStatusList() {
    this.stopPaymentSvc.getStatusList().subscribe({
      next: (res: any) => {
        this.status = res;
        this.selectedStatus = res?.[0]?.TrStatusID;
      },
      error: () => {
        this.status = [];
      },
      complete: () => {},
    });
  }

  getAllStatusList() {
    this.stopPaymentSvc.getAllStatusList().subscribe({
      next: (res: any) => {
        this.allStatus = res;
        this.selectedAllStatus = res?.[0]?.TrStatusID;
      },
      error: () => {
        this.allStatus = [];
      },
      complete: () => {},
    });
  }

  approveSelected() {
    const selectedTransactions = this.AllData.filter(
      (transaction: any) => transaction.selected
    );
    if (selectedTransactions.length === 0) {
      alert('No transactions selected for approval.');
      return;
    }

    const payload = selectedTransactions.map((transaction: any) => ({
      SRTRNo: transaction.SRTRNo,
      TrStatusID: transaction.TrStatusID, // Assuming status ID 11 means "Approved"
      EnteredBy: '1001005', // You can replace this with the appropriate value
      IsChk: true,
    }));

    this.stopPaymentSvc.patchApprovedTransaction(payload).subscribe({
      next: (response: any) => {
        console.log('Transactions approved:', response);
        alert('Selected transactions have been approved.');
        this.getAllData(); // Refresh data after approval
      },
      error: (error: any) => {
        // alert('An error occurred while approving the transactions.');
        this.toastrSvc.error(error?.error?.message);
      },
    });
  }

  getAgentTypeList() {
    this.stopPaymentSvc.getAgentTypeList().subscribe({
      next: (res: any) => {
        this.agentTypes = res;
        this.selectedAgentType = res?.[0]?.Group_Id;
      },
      error: () => {
        this.agentTypes = [];
      },
      complete: () => {},
    });
  }

  getSendCountryList() {
    this.stopPaymentSvc.getSendCountryList().subscribe({
      next: (res: any) => {
        this.sendCountryList = res;
      },
      error: () => {
        this.sendCountryList = [];
      },
      complete: () => {},
    });
  }

  onSendCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.selectedSendCurrencyList = 'All';
      this.selectedDestCurrencyList = 'All';
      this.selectedAgentBranchList = 'All';
      this.selectedUserList = 'All';
      this.getSendCurrencyList(countryId);
      this.getAgentList(countryId);
    }
  }

  getDestinationCountryList() {
    this.stopPaymentSvc.getDestinationCountryList().subscribe({
      next: (res: any) => {
        this.destCountryList = res;
      },
      error: () => {
        this.destCountryList = [];
      },
      complete: () => {},
    });
  }

  onDestinationCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.getDestinationCurrencyList(countryId);
    }
  }

  getSendCurrencyList(countryId: any = 0) {
    this.stopPaymentSvc.getSendCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.sendCurrencyList = res;
      },
      error: () => {
        this.sendCurrencyList = [];
      },
      complete: () => {},
    });
  }

  getDestinationCurrencyList(countryId: any = 0) {
    this.stopPaymentSvc.getDestinationCurrencyList(countryId).subscribe({
      next: (res: any) => {
        this.destCurrencyList = res;
      },
      error: () => {
        this.destCurrencyList = [];
      },
      complete: () => {},
    });
  }

  getAgentList(targetCountryId: any = 0) {
    this.stopPaymentSvc.getAgentList(targetCountryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: () => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId: any) {
    if (agentId != 'All') {
      this.getAgentBranchList(agentId);
    }
  }

  getUserList(agentId: any = 0, agentBranchId: any = 0) {
    this.stopPaymentSvc.getUserList(agentId, agentBranchId).subscribe({
      next: (res: any) => {
        this.userList = res;
      },
      error: () => {
        this.userList = [];
      },
      complete: () => {},
    });
  }

  getAgentBranchList(agentId: any = 0) {
    this.stopPaymentSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
      },
      error: () => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  onAgentBranchChange(agentBranchId: any) {
    if (agentBranchId != 'All' && this.selectedAgentList != 'All') {
      this.userList(this.selectedAgentList, agentBranchId);
    }
  }

  getTransferTypeList() {
    this.stopPaymentSvc.getTransferTypeList().subscribe({
      next: (res: any) => {
        this.transferTypeList = res;
      },
      error: () => {
        this.transferTypeList = [];
      },
      complete: () => {},
    });
  }

  onSearchByChange(event) {
    this.searchByType = Number(event.target.value);
  }

  onSelectViewLast(event) {
    this.viewLast = event;
  }

  onSelectStatus(event) {
    this.selecteStatus = event;
  }
  getAllData() {
    if (this.searchByType == 1 && !this.transactionNo) {
      alert('Please Enter Transaction No.');
      return;
    }
    this.isLoading = true;
    let payload: any = {};
    if (this.searchByType == 1) {
      payload = {
        SearchByType: this.searchByType,
        TopRecord: 0,
        Agent_ID: 0,
        PayOutCountry: -1,
        PayOutCurrency: -1,
        SubType: -1,
        SessionAgent_ID: 0,
        SRTRNo: this.transactionNo,
        TrStatusID: -1,
        TrTypeID: -1,
        PayInCountry: -1,
        PayInCurrency: -1,
        LoginCountryID: 0,
        UserID: -1,
        AgentBranchId: -1,
      };
    }
    if (this.searchByType === 2) {
      payload = {
        SearchByType: this.searchByType,
        TopRecord: this.viewLast,
        Agent_ID: this.selectedAgentList,
        PayOutCountry: this.selectedDestCountryList,
        PayOutCurrency: this.selectedDestCurrencyList,
        SubType: -1,
        TrStatusID: this.selecteStatus,
        TrStatusList: this.status,
        TrTypeID: this.selectedType,
        PayInCountry: this.selectedSendCountryList,
        PayInCurrency: this.selectedSendCurrencyList,
        UserID: getCacheData('G_UserID'),
        AgentBranchId: this.selectedAgentBranchList,
      };
    }
    if (this.searchByType === 3) {
      payload = {
        SearchByType: this.searchByType,
        TopRecord: this.viewLast,
        Agent_ID: 0,
        PayOutCountry: -1,
        PayOutCurrency: -1,
        SubType: -1,
        SRTRNo: this.transactionNo,
        TrStatusID: this.selecteStatus,
        TrStatusList: this.status,
        TrTypeID: -1,
        StartDate: this.startDate,
        EndDate: this.endDate,
        PayInCountry: -1,
        PayInCurrency: -1,
        UserID: '-1',
        AgentBranchId: -1,
        Beneficiary_Name_F: this.beneficiaryNameFirst
          ? this.beneficiaryNameFirst
          : '',
        Beneficiary_Name_L: this.beneficiaryNameLast
          ? this.beneficiaryNameLast
          : '',
        Customer_Name_F: this.customerNameFirst ? this.customerNameFirst : '',
        Customer_Name_L: this.customerNameLast ? this.customerNameLast : '',
      };
    }

    this.stopPaymentSvc.getAllData(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        // Add 'selected' property to each data item
        this.AllData = res.map((item: any) => ({ ...item, selected: false }));
      },
      error: () => {
        this.transferTypeList = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  clearCriteria() {
    this.transactionNo = '';
    this.startDate = '';
    this.endDate = '';
    this.beneficiaryNameFirst = '';
    this.beneficiaryNameLast = '';
    this.customerNameFirst = '';
    this.customerNameLast = '';
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  onEdit(stopPayment: any) {
    if (stopPayment) {
      this.router.navigate([
        '/panel/edit-stop-payment',
        stopPayment?.Ex_Branch_ID,
        stopPayment?.App_ID,
        stopPayment?.Reference_No,
        stopPayment?.SRTRNo,
      ]);
    }
  }
}
