import { Component, ElementRef, TemplateRef, ViewChild } from '@angular/core';
import { TransactionAccountingDetailsService } from '../services/transaction-accounting-details.service';
import * as XLSX from 'xlsx';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';

@Component({
  selector: 'app-transaction-accounting-details',
  templateUrl: './transaction-accounting-details.component.html',
  styleUrls: ['./transaction-accounting-details.component.css'],
})
export class TransactionAccountingDetailsComponent {
  @ViewChild('accountingTable') accountingTable: ElementRef;
  @ViewChild('transactionAccountModal') transactionAccountModal: TemplateRef<any>;
  bsModalRef: BsModalRef;

  countryList: any = [];
  selectedCountryList: any = '-1';
  agentList: any = [];
  selectedAgentList: any = '-1';
  fromDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: any = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  CustomerType: any = '-1';
  transactionNo: any = '';
  transactionNoModel: any = '';
  allAccountingDetails: any = [];
  selectedFileFormat: any = '1';
  isLoading: boolean = false;
  selectedTransaction: any = null;

  constructor(
    private accountingDetailSvc: TransactionAccountingDetailsService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentListByCountry(0);
  }

  getCountryList() {
    this.accountingDetailSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != -1) {
      this.getAgentListByCountry(countryId);
    }
  }

  getAgentListByCountry(countryId: any = 0) {
    this.accountingDetailSvc.getAgentListByCountry(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        if (countryId == 0) {
          this.selectedAgentList = res?.[0]?.Agent_ID;
        }
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }

    if (!this.fromDate) {
      alert('Please Select From Date!');
      return;
    }

    if (!this.toDate) {
      alert('Please Select To Date!');
      return;
    }

    let payload: any = {
      FromDate: this.formatDate(this.fromDate),
      ToDate: this.formatDate(this.toDate),
      CountryID: this.selectedCountryList,
      AgentID: this.selectedAgentList,
      CustomerID: Number(this.CustomerType),
      TrxnNo: '',
    };

    this.isLoading = true;
    this.accountingDetailSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allAccountingDetails = res;
      },
      error: (error) => {
        this.isLoading = false;

        alert(error.error.message)
        this.allAccountingDetails = [];
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  formatDate(date: Date): string {
    const d = new Date(date);
    const month = '' + (d.getMonth() + 1);
    const day = '' + d.getDate();
    const year = d.getFullYear();
    return [
      month.length < 2 ? '0' + month : month,
      day.length < 2 ? '0' + day : day,
      year,
    ].join('/');
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Fill Transaction Number!');
      return;
    }

    this.isLoading = true;
    let payload: any = {
      TrxnNo: this.transactionNo,
    };

    this.accountingDetailSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allAccountingDetails = res;
      },
      error: () => {
        this.isLoading = false;
        this.allAccountingDetails = [];
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.accountingTable.nativeElement);

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Accounting_Details${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  openTransactionAccountModal(transaction: any) {
    this.transactionNoModel = transaction;
    this.accountingDetailSvc.getTransactionDetails(transaction).subscribe({
      next: (res: any) => {
        if (res && res[0].TransactionDetailsReportSentAccount) {
          this.selectedTransaction = res[0].TransactionDetailsReportSentAccount;
        } else {
          this.selectedTransaction = [];
        }
      },
      error: (err) => {
        this.selectedTransaction = [];
      },
      complete: () => {},
    });

    this.bsModalRef = this.modalService.show(this.transactionAccountModal, {
      class: 'modal-lg',
    });
  }
}
