import { Component } from '@angular/core';
import { ViewPendingApprovalService } from '../view-pending-approval.service';
import { getCacheData } from '../utils/helper';
import { AuthService } from '../services/auth.service';

@Component({
  selector: 'app-view-pending-approval',
  templateUrl: './view-pending-approval.component.html',
  styleUrls: ['./view-pending-approval.component.css'],
})
export class ViewPendingApprovalComponent {
  TransactionReport: any = [];
  UserList: any;
  AgentList: any;
  DestCurrencyList: any;
  status: any;
  searchByType: any = 1;
  AgentTypeList: any;
  startDate: any;
  endDate: any;
  AgentID: any;
  SendCountryList: any;
  PayoutCountryList: any;
  SendCurrencyList: any;
  AgentBranchList: any;
  TransactionNumber: any;
  customerNameFirst: any;
  customerNameLast: any;
  beneficiaryNameFirst: any;
  beneficiaryNameLast: any;
  viewLast: any = 10;
  selectedStatus: any;
  selectedAgentType: any;
  selectedSendCountry: any = -1;
  selectedDestCountry: any = -1;
  selectedSendCurrency: any = -1;
  selectedDestCurrency: any = -1;
  selectedAgent: any = -1;
  selectedAgentBranch: any = -1;
  selectedUser: any = -1;

  constructor(
    private ViewPendingApprovalSvc: ViewPendingApprovalService,
    private authSvc: AuthService
  ) {}

  ngOnInit(): void {
    this.getStatus();
    this.getAgentTypeList();
    this.getSendCountryList();
    this.getPayoutCountryList();
    this.startDate = this.authSvc.getTodayDate();
    this.endDate = this.authSvc.getTodayDate();
  }

  onSelectSearchFilter(No) {
    this.searchByType = No;
  }

  getStatus() {
    this.ViewPendingApprovalSvc.getStatus().subscribe((res) => {
      this.status = res;
      this.selectedStatus = res?.[0]?.TrStatusID;
    });
  }

  getAgentTypeList() {
    this.ViewPendingApprovalSvc.getAgentTypeList().subscribe((res) => {
      this.AgentTypeList = res;
      this.selectedAgentType = res?.[0]?.Group_Id;
    });
  }

  getSendCountryList() {
    this.ViewPendingApprovalSvc.getSendCountryList().subscribe((res) => {
      this.SendCountryList = res;
    });
  }
  onSelectCountry(CountryID) {
    this.getAgentList(CountryID);
    this.getSendCurrencyList(CountryID);
  }

  getSendCurrencyList(CountryID) {
    this.ViewPendingApprovalSvc.getSendCurrencyList(CountryID).subscribe(
      (res) => {
        this.SendCurrencyList = res;
      }
    );
  }

  getPayoutCountryList() {
    this.ViewPendingApprovalSvc.getPayoutCountryList().subscribe((res) => {
      this.PayoutCountryList = res;
    });
  }

  onSelectPayoutCountry(CountryID) {
    this.getDestCurrencyList(CountryID);
  }

  getDestCurrencyList(CountryID) {
    this.ViewPendingApprovalSvc.getDestCurrencyList(CountryID).subscribe(
      (res) => {
        this.DestCurrencyList = res;
      }
    );
  }

  getAgentList(CountryID) {
    this.ViewPendingApprovalSvc.getAgentList(CountryID).subscribe((res) => {
      this.AgentList = res;
    });
  }

  onSelectAgent(AgentID) {
    this.AgentID = AgentID;
    this.getAgentBranchList(AgentID);
  }

  getAgentBranchList(AgentID) {
    this.ViewPendingApprovalSvc.getAgentBranchList(AgentID).subscribe((res) => {
      this.AgentBranchList = res;
    });
  }

  onSelectAgentBranch(AgentBranchID) {
    this.getUserList(AgentBranchID);
  }

  getUserList(AgentBranchID: any) {
    this.ViewPendingApprovalSvc.getUserList(
      this.AgentID,
      AgentBranchID
    ).subscribe((res) => {
      this.UserList = res;
    });
  }

  getTransactionPendingDetail() {
    if (this.searchByType == 1 && !this.TransactionNumber) {
      alert('Please Enter Transaction No.');
      return;
    }

    if (this.searchByType == 3 && (!this.startDate || !this.endDate)) {
      alert('Please Select Start & End Dates.');
      return;
    }

    const commonPayload = {
      TopRecord: this.viewLast,
      SRTRNo: '',
      TrStatusID: -1,
      TrStatusList: [
        { Id: -1, Value: 'All' },
        { Id: 5, Value: 'AgentBlocked' },
        { Id: 7, Value: 'Cancellation request is pending or in process' },
        { Id: 4, Value: 'ComplianceBlock' },
        { Id: 8, Value: 'Hold' },
        { Id: 12, Value: 'Old' },
        { Id: 3, Value: 'Paid' },
        { Id: 2, Value: 'UnPaid' },
      ],
      SearchByType: this.searchByType ? this.searchByType : 1,
      UserGroupType: 0,
      UserID: getCacheData('G_UserID'),
      AgentBranchId: 0,
      LoginAgentCountryID: 0,
      LoginAgentCityID: 0,
      LoginAgentLocationID: 0,
      Agent_ID: 0,
      SessionAgent_ID: 0,
      Customer_Name_F: '',
      Customer_Name_L: '',
      Beneficiary_Name_F: '',
      Beneficiary_Name_L: '',
      TrTypeID: -1,
      PayInCountry: -1,
      PayInCurrency: -1,
      PayOutCountry: -1,
      PayOutCurrency: -1,
      StartDate: '',
      EndDate: '',
      Benef_Phone_No: '',
      Remitter_Phone_No: '',
    };

    let payload = { ...commonPayload };

    switch (this.searchByType) {
      case 1:
        payload.SRTRNo = this.TransactionNumber;
        payload.AgentBranchId = 0;
        payload.Agent_ID = 0;
        break;
      case 2:
        payload.TrStatusID = this.selectedStatus;
        payload.AgentBranchId = this.selectedAgentBranch;
        payload.Agent_ID = this.selectedAgent;
        payload.PayInCountry = this.selectedSendCountry;
        payload.PayInCurrency = this.selectedSendCurrency;
        payload.PayOutCountry = this.selectedDestCountry;
        payload.PayOutCurrency = this.selectedDestCurrency;
        break;
      case 3:
        payload.StartDate = this.startDate;
        payload.EndDate = this.endDate;
        payload.Customer_Name_F = this.customerNameFirst
          ? this.customerNameFirst
          : '';
        payload.Customer_Name_L = this.customerNameLast
          ? this.customerNameLast
          : '';
        payload.Beneficiary_Name_F = this.beneficiaryNameFirst
          ? this.beneficiaryNameFirst
          : '';
        payload.Beneficiary_Name_L = this.beneficiaryNameLast
          ? this.beneficiaryNameLast
          : '';
        break;
    }

    this.ViewPendingApprovalSvc.getTransactionPendingDetail(payload).subscribe(
      (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.TransactionReport = res;
      }
    );
  }

  clearCriteria() {
    this.TransactionNumber = '';
    this.startDate = '';
    this.endDate = '';
    this.beneficiaryNameFirst = '';
    this.beneficiaryNameLast = '';
    this.customerNameFirst = '';
    this.customerNameLast = '';
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  aprovedTransaction() {
    const selectedRecords = this.TransactionReport.filter(
      (record) => record.selected
    );
    if (selectedRecords?.length == 0) {
      alert('Please Select Transaction for Approved.');
      return;
    }
    const payload = {
      UserGroupID: getCacheData('G_UserGroup'), // Replace with actual value if needed
      UserID: getCacheData('G_UserID'),
      Ex_Branch_ID: selectedRecords.map((record) => ({
        Id: 0,
        Value: record.Ex_Branch_ID, // Replace with actual value if needed
      })),
      App_ID: selectedRecords.map((record) => ({
        Id: 0, // If there's a specific Id, replace 0 with that value
        Value: record.App_ID,
      })),
    };
    this.ViewPendingApprovalSvc.approveTransaction(payload).subscribe({
      next: (res: any) => {
        if (res) {
          alert('Transactions has been approved');
        }
      },
      error: (err) => {
        alert(err?.error?.message);
      },
      complete: () => {},
    });
  }
}
